#!/usr/bin/env python
# -*- coding: utf-8 -*-
from string import ascii_letters
from random import choice

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.test import TestCase, override_settings

from ..models import Tag, Entry


@override_settings(SITE_ID=1)
@override_settings(ROOT_URLCONF='blargg.tests.urls')
class TestViews(TestCase):

    def setUp(self):
        username = ''.join([choice(ascii_letters) for i in range(10)])
        User = get_user_model()
        user = User.objects.create(
            username=username,
            password='{0}@example.com'.format(username)
        )

        # Sample Entry
        self.entry = Entry(
            site=Site.objects.get(pk=settings.SITE_ID),
            author=user,
            title="Test Entry",
            raw_content="Test Content",
            tag_string="foo, bar"
        )
        self.entry.publish()  # Calls .save()

        # Sample Tag (should have foo, bar tags from entry's tag_string)
        self.tag = Tag.objects.get(name="foo")

    def test_list_tags(self):
        url = reverse('blargg:list_tags')
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)
        self.assertIn('object_list', resp.context)
        self.assertEqual(len(resp.context['object_list']), 2)

    def test_tagged_entry_list(self):
        url = reverse('blargg:tagged_entry_list', args=[self.tag.slug])
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)
        self.assertIn('object_list', resp.context)
        self.assertEqual(len(resp.context['object_list']), 1)

    def test_entry_archive_day(self):
        # NOTE: Entries are stored in UTC and the EntryDayArchiveView converts
        # dates to the local timezone (if USE_TZ=True).
        # Also, Entry.get_absolute_url also converts to TIME_ZONE if USE_TZ is
        # True.
        y, m, d = self.entry.published_on.strftime("%Y-%m-%d").split("-")
        url = reverse('blargg:entry_archive_day', args=[y, m, d])
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)
        self.assertIn('object_list', resp.context)
        self.assertTemplateUsed("blargg/entry_archive_day.html")

    def test_entry_archive_month(self):
        y, m, d = self.entry.published_on.strftime("%Y-%m-%d").split("-")
        url = reverse('blargg:entry_archive_month', args=[y, m])
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)
        self.assertIn('object_list', resp.context)
        self.assertIn('date_list', resp.context)
        self.assertEqual(len(resp.context['object_list']), 1)
        self.assertEqual(len(resp.context['date_list']), 1)
        self.assertTemplateUsed("blargg/entry_archive_month.html")

    def test_entry_archive_year(self):
        y, m, d = self.entry.published_on.strftime("%Y-%m-%d").split("-")
        url = reverse('blargg:entry_archive_year', args=[y])
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)
        self.assertIn('date_list', resp.context)
        self.assertEqual(len(resp.context['date_list']), 1)
        self.assertTemplateUsed("blargg/entry_archive_year.html")

    def test_entry_detail_with_date(self):
        y, m, d = self.entry.published_on.strftime("%Y-%m-%d").split("-")
        url = reverse('blargg:entry_detail', args=[y, m, d, self.entry.slug])
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)
        self.assertIn('object', resp.context)
        self.assertIsInstance(resp.context['object'], Entry)
        self.assertTemplateUsed("blargg/entry_detail.html")

    def test_entry_detail_without_date(self):
        url = reverse('blargg:entry_detail', args=[self.entry.slug])
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)
        self.assertIn('object', resp.context)
        self.assertIsInstance(resp.context['object'], Entry)
        self.assertTemplateUsed("blargg/entry_detail.html")

    def test_list_entries(self):
        """Tests the ARchiveIndexView."""
        url = reverse('blargg:list_entries')
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)
        self.assertIn('object_list', resp.context)
        self.assertEqual(len(resp.context['object_list']), 1)
        self.assertTemplateUsed("blargg/entry_list.html")



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-01-07 23:16
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='Entry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=256)),
                ('raw_content', models.TextField(help_text='Content entered by the author.')),
                ('content_format', models.CharField(choices=[('html', 'HTML'), ('rst', 'reStructured Text'), ('md', 'Markdown (not yet supported)')], max_length=4)),
                ('rendered_content', models.TextField(editable=False)),
                ('published', models.BooleanField(default=False, help_text='Show this entry to the public')),
                ('slug', models.SlugField(help_text='A slug/url used to identify this entry.', max_length=255, unique=True)),
                ('date_slug', models.SlugField(editable=False, help_text='A slug/url used to identify this entry; also includes the date on which the entry was published.', max_length=255, unique=True)),
                ('tag_string', models.TextField(blank=True, help_text='A comma-separated list of tags.')),
                ('published_on', models.DateTimeField(blank=True, editable=False, null=True)),
                ('updated_on', models.DateTimeField(auto_now=True)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.Site')),
            ],
            options={
                'verbose_name': 'Entry',
                'get_latest_by': 'published_on',
                'ordering': ['-published_on', 'title'],
                'verbose_name_plural': 'Entries',
            },
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=256)),
                ('slug', models.SlugField(editable=False, max_length=256, unique=True)),
            ],
            options={
                'verbose_name': 'Tag',
                'ordering': ['name'],
                'verbose_name_plural': 'Tags',
            },
        ),
        migrations.AddField(
            model_name='entry',
            name='tags',
            field=models.ManyToManyField(editable=False, to='blargg.Tag'),
        ),
    ]



import pytz
import re

from collections import Counter
try:
    from docutils.core import publish_parts as docutils_publish
    assert docutils_publish  # placate flake8
except ImportError:  # pragma: no cover
    docutils_publish = None  # pragma: no cover

try:
    from markdown import markdown
    assert markdown  # placate flake8
except ImportError:  # pragma: no cover
    markdown = None  # pragma: no cover

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.template.defaultfilters import slugify
from django.utils.html import strip_tags
from django.utils.safestring import mark_safe
from django.utils.timezone import now as utc_now
from django.utils.timezone import make_naive

from .signals import entry_published


class TagManager(models.Manager):
    def create_tags(self, entry):
        """Inspects an ``Entry`` instance, and builds associates ``Tag``
        objects based on the values in the ``Entry``'s ``tag_string``."""
        tag_list = [t.lower().strip() for t in entry.tag_string.split(',')]
        for t in tag_list:
            tag, created = self.get_or_create(name=t)
            entry.tags.add(tag)


class Tag(models.Model):
    """A *really* light-weight tagging class."""
    name = models.CharField(max_length=256)
    slug = models.SlugField(max_length=256, editable=False, unique=True)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ['name']
        verbose_name = 'Tag'
        verbose_name_plural = 'Tags'

    def save(self, *args, **kwargs):
        """Generate a unique slug for each tag."""
        self.name = self.name.lower().strip()
        self.slug = slugify(self.name)
        super(Tag, self).save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse('blargg:tagged_entry_list', args=[self.slug])

    objects = TagManager()


class Entry(models.Model):
    CONTENT_FORMAT_CHOICES = (
        ('html', 'HTML'),
        ('rst', 'reStructured Text'),
        ('md', 'Markdown (not yet supported)'),
    )

    site = models.ForeignKey(Site)
    author = models.ForeignKey(settings.AUTH_USER_MODEL)
    title = models.CharField(max_length=256)

    raw_content = models.TextField(help_text="Content entered by the author.")
    content_format = models.CharField(
        max_length=4,
        choices=CONTENT_FORMAT_CHOICES
    )
    rendered_content = models.TextField(editable=False)
    published = models.BooleanField(
        default=False,
        blank=True,
        help_text="Show this entry to the public"
    )
    slug = models.SlugField(
        max_length=255,
        unique=True,
        help_text="A slug/url used to identify this entry."
    )
    date_slug = models.SlugField(
        max_length=255,
        unique=True,
        editable=False,
        help_text="A slug/url used to identify this entry; also includes "
                  "the date on which the entry was published."
    )

    tag_string = models.TextField(
        blank=True,
        help_text="A comma-separated list of tags."
    )
    tags = models.ManyToManyField(Tag, editable=False)

    published_on = models.DateTimeField(blank=True, null=True, editable=False)
    updated_on = models.DateTimeField(auto_now=True)
    created_on = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.title

    class Meta:
        ordering = ['-published_on', 'title']
        get_latest_by = 'published_on'
        verbose_name = 'Entry'
        verbose_name_plural = 'Entries'

    def _create_slug(self):
        """Generates a slug from the Title."""
        # Don't overwrite any existing slugs by default.
        if not self.slug:
            self.slug = slugify(self.title)

    def _create_date_slug(self):
        """Prefixes the slug with the ``published_on`` date."""
        if not self.pk:
            # haven't saved this yet, so use today's date
            d = utc_now()
        elif self.published and self.published_on:
            # use the actual published on date
            d = self.published_on
        elif self.updated_on:
            # default to the last-updated date
            d = self.updated_on
        self.date_slug = u"{0}/{1}".format(d.strftime("%Y/%m/%d"), self.slug)

    def _render_content(self):
        """Renders the content according to the ``content_format``."""
        if self.content_format == "rst" and docutils_publish is not None:
            doc_parts = docutils_publish(
                source=self.raw_content,
                writer_name="html4css1"
            )
            self.rendered_content = doc_parts['fragment']
        elif self.content_format == "rs" and docutils_publish is None:
            raise RuntimeError("Install docutils to pubilsh reStructuredText")
        elif self.content_format == "md" and markdown is not None:
            self.rendered_content = markdown(self.raw_content)
        elif self.content_format == "md" and markdown is None:
            raise RuntimeError("Install Markdown to pubilsh markdown")
        else:  # Assume we've got html
            self.rendered_content = self.raw_content

    def _set_published(self):
        """Set the fields that need to be set in order for this thing to
        appear "Published", and send the ``entry_published`` signal."""
        self.published = True
        self.published_on = utc_now()
        return True

    def save(self, *args, **kwargs):
        """Auto-generate a slug from the name."""
        self._create_slug()
        self._create_date_slug()
        self._render_content()

        # Call ``_set_published`` the *first* time this Entry is published.
        # NOTE: if this is unpublished, and then republished, this method won't
        # get called; e.g. the date won't get changed and the
        # ``entry_published`` signal won't get re-sent.
        send_published_signal = False
        if self.published and self.published_on is None:
            send_published_signal = self._set_published()

        super(Entry, self).save(*args, **kwargs)

        # We need an ID before we can send this signal.
        if send_published_signal:
            entry_published.send(sender=self, entry=self)

    def get_absolute_url(self):
        """URL based on the entry's slug."""
        return reverse('blargg:entry_detail', args=[self.slug])

    def get_absolute_url_with_date(self):
        """URL based on the entry's date & slug."""
        pub_date = self.published_on

        if pub_date and settings.USE_TZ:
            # If TZ is enabled, convert all of these dates from UTC to whatever
            # the project's timezone is set as. Ideally, we'd pull this form
            # some user settings, but the *canonical* publish time is that of
            # the author (asssuming author == owner of this project).
            pub_date = make_naive(pub_date, pytz.utc)  # Make naive
            pub_date = pytz.timezone(settings.TIME_ZONE).localize(pub_date)
        if pub_date:
            args = [
                pub_date.strftime("%Y"),
                pub_date.strftime("%m"),
                pub_date.strftime("%d"),
                self.slug
            ]
        else:
            args = [self.slug]
        return reverse('blargg:entry_detail', args=args)

    def publish(self):
        """Puplish & Save."""
        self._set_published()
        self.save()

    def unpublish(self):
        self.published = False
        self.published_on = None
        self.save()

    @property
    def content(self):
        safe_content = mark_safe(self.rendered_content)
        return safe_content

    @property
    def crossposted_content(self):
        """Content to be cross-posted on other sites. This method adds an
        additional line of content with a link back to the original site as
        well as the `#end` tag. This signals the end blog content according
        to: https://support.google.com/blogger/answer/41452?hl=en

        """
        url = "http://{0}{1}".format(self.site.domain, self.get_absolute_url())
        origin = (
            '<p><em>This entry was originally published at: '
            '<a href="{0}">{1}</a>.</em></p>#end'.format(url, url)
        )
        return mark_safe(u"{0}{1}".format(self.content, origin))


@receiver(post_save, sender=Entry, dispatch_uid='generate-entry-tags')
def generate_entry_tags(sender, instance, created, raw, using, **kwargs):
    """Generate the M2M ``Tag``s for an ``Entry`` right after it has
    been saved."""
    Tag.objects.create_tags(instance)


def entry_stats(entries, top_n=10):
    """Calculates stats for the given ``QuerySet`` of ``Entry``s."""

    wc = Counter()  # A Word counter
    for content in entries.values_list("rendered_content", flat=True):
        # Do a little cleanup
        content = strip_tags(content)  # remove all html tags
        content = re.sub('\s+', ' ', content)  # condense all whitespace
        content = re.sub('[^A-Za-z ]+', '', content)  # remove non-alpha chars
        wc.update(content.split())

    return {
        "total_words": len(wc.values()),
        "most_common": wc.most_common(top_n),
    }



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Tag'
        db.create_table(u'blargg_tag', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=256)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=256)),
        ))
        db.send_create_signal(u'blargg', ['Tag'])

        # Adding model 'Entry'
        db.create_table(u'blargg_entry', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('site', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['sites.Site'])),
            ('author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=256)),
            ('raw_content', self.gf('django.db.models.fields.TextField')()),
            ('content_format', self.gf('django.db.models.fields.CharField')(max_length=4)),
            ('rendered_content', self.gf('django.db.models.fields.TextField')()),
            ('published', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=255)),
            ('date_slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=255)),
            ('tag_string', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('published_on', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('updated_on', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('created_on', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal(u'blargg', ['Entry'])

        # Adding M2M table for field tags on 'Entry'
        db.create_table(u'blargg_entry_tags', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('entry', models.ForeignKey(orm[u'blargg.entry'], null=False)),
            ('tag', models.ForeignKey(orm[u'blargg.tag'], null=False))
        ))
        db.create_unique(u'blargg_entry_tags', ['entry_id', 'tag_id'])


    def backwards(self, orm):
        # Deleting model 'Tag'
        db.delete_table(u'blargg_tag')

        # Deleting model 'Entry'
        db.delete_table(u'blargg_entry')

        # Removing M2M table for field tags on 'Entry'
        db.delete_table('blargg_entry_tags')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'blargg.entry': {
            'Meta': {'ordering': "['-published_on', 'title']", 'object_name': 'Entry'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"}),
            'content_format': ('django.db.models.fields.CharField', [], {'max_length': '4'}),
            'created_on': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'published_on': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'raw_content': ('django.db.models.fields.TextField', [], {}),
            'rendered_content': ('django.db.models.fields.TextField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['sites.Site']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255'}),
            'tag_string': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['blargg.Tag']", 'symmetrical': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'updated_on': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        u'blargg.tag': {
            'Meta': {'ordering': "['name']", 'object_name': 'Tag'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '256'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['blargg']


from django.conf.urls import url
from django.views.generic import ArchiveIndexView
from django.views.generic import ListView

from .models import Entry, Tag
from .views import EntryDayArchiveView
from .views import EntryDetailView
from .views import EntryMonthArchiveView
from .views import EntryYearArchiveView
from .views import TaggedEntryListView

# URL examples
# ------------
# /blog/tags/                   -- tag list detail
# /blog/tags/foo/               -- entries tagged with "foo"
# /blog/a-sample-entry/         -- entry detail
# /blog/2013/01/05/             -- entry list (by year, month, day)
# /blog/2013/01/                -- entry list (by year, month)
# /blog/2013/                   -- entry list (by year)
# /blog/2013/05/a-sample-entry/ -- entry detail (with date slug)
# /blog/                        -- entry detail (latest published post)

urlpatterns = [
    url(r'^tags/$', ListView.as_view(model=Tag), name='list_tags'),
    url(
        r'^tags/(?P<tag_slug>.*)/$',
        TaggedEntryListView.as_view(),
        name='tagged_entry_list'
    ),

    # Year, Month, Day Archives
    url(
        r'^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/$',
        EntryDayArchiveView.as_view(),
        name='entry_archive_day'
    ),
    url(
        r'^(?P<year>\d{4})/(?P<month>\d{2})/$',
        EntryMonthArchiveView.as_view(),
        name='entry_archive_month'
    ),
    url(
        r'^(?P<year>\d{4})/$',
        EntryYearArchiveView.as_view(),
        name='entry_archive_year'
    ),

    # Detail views (with and without the date)
    url(
        r'^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/(?P<slug>.*)/$',
        EntryDetailView.as_view(),
        name='entry_detail'
    ),
    url(
        r'^(?P<slug>.*)/$',
        EntryDetailView.as_view(),
        name='entry_detail'
    ),
    url(
        r'^$',
        ArchiveIndexView.as_view(
            model=Entry,
            date_field='published_on',
            paginate_by=10
        ),
        name='list_entries'
    ),
]



from django.views.generic import DetailView, ListView
from django.views.generic import DayArchiveView, MonthArchiveView
from django.views.generic import YearArchiveView
from django.views.generic.list import MultipleObjectMixin

from .models import Entry, entry_stats


class EntryStatsMixin(MultipleObjectMixin):
    """This mixin will calculate entry stats (counting words) for a View's
    queryset of objects."""

    def get_context_data(self, **kwargs):
        context = super(EntryStatsMixin, self).get_context_data(**kwargs)

        # Calculate stats for the queryset of Entries & add to the context.
        objects = context['object_list']
        if not objects.exists() and 'date_list' in context and context['date_list']:
            # EntryYearArchiveViews won't include any objects, so query for
            # thos using the year
            years = list(set(dt.year for dt in context['date_list']))
            if len(years) > 0:
                objects = Entry.objects.filter(published_on__year=years[0])

        context.update(entry_stats(objects))
        return context


class TaggedEntryListView(ListView):
    """List all ``Entry``s that have the given ``Tag``(s). Mulitple ``Tag``s
    may be separated by a plus; For example: /blog/tags/foo+bar would retrieve
    all ``Entry``s tagged with both "foo" and "bar".

    """
    allow_empty = True
    model = Entry
    tags = None

    def get_queryset(self):
        tag_list = self.kwargs['tag_slug'].split('+')
        self.tags = filter(lambda x: len(x) > 0, tag_list)
        return Entry.objects.filter(tags__slug__in=self.tags).distinct()

    def get_context_data(self, **kwargs):
        context = super(TaggedEntryListView, self).get_context_data(**kwargs)
        context['tags'] = self.tags
        return context


class EntryDetailView(DetailView):
    """Detail for an ``Entry``."""
    model = Entry
    slug_field = 'slug'


# Year, Month, Day Archives
# -------------------------

class EntryYearArchiveView(EntryStatsMixin, YearArchiveView):
    queryset = Entry.objects.filter(published=True)
    date_field = "published_on"
    year_format = '%Y'
    template_name = "blargg/entry_archive_year.html"


class EntryMonthArchiveView(EntryStatsMixin, MonthArchiveView):
    queryset = Entry.objects.filter(published=True)
    date_field = "published_on"
    year_format = '%Y'
    month_format = "%m"
    template_name = "blargg/entry_archive_month.html"


class EntryDayArchiveView(EntryStatsMixin, DayArchiveView):
    # NOTE: Entries are stored in UTC and this view converts dates to the
    # local timezone (if USE_TZ=True). Therefore, Entry.get_absolute_url also
    # converts to TIME_ZONE if USE_TZ=True.
    queryset = Entry.objects.filter(published=True)
    date_field = "published_on"
    year_format = '%Y'
    month_format = "%m"
    day_format = "%d"
    template_name = "blargg/entry_archive_day.html"
    allow_empty = True



"""
This module contains classes for both RSS and Atom feeds. To use these, you'll
need to set up a couple of URL patterns; e.g.

    from blargg.feeds import RSSEntriesFeed, AtomEntriesFeed

    urlpatterns = patterns('',
        (r'^feed/rss/$', RSSEntriesFeed()),
        (r'^feed/atom/$', AtomEntriesFeed()),
    )

"""

from django.contrib.syndication.views import Feed
from django.utils.feedgenerator import Atom1Feed

from .models import Entry


class RSSEntriesFeed(Feed):
    """An RSS feed for all ``Entry``'s"""
    title = "brad's blog"
    link = "/blog/"
    description = "Entries from brad's blog"

    def items(self):
        return Entry.objects.filter(published=True)

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.rendered_content


class AtomEntriesFeed(RSSEntriesFeed):
    feed_type = Atom1Feed
    subtitle = RSSEntriesFeed.description



from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.test import TestCase, override_settings

from ..feeds import RSSEntriesFeed, AtomEntriesFeed
from ..models import Entry


@override_settings(SITE_ID=1)
class TestRSSEntriesFeed(TestCase):

    def setUp(self):
        # Some setup (need a User and an Entry)
        User = get_user_model()
        username = 'blargg'
        user = User.objects.create(
            username=username,
            password='{0}@example.com'.format(username)
        )
        self.entry = Entry(
            site=Site.objects.get(pk=settings.SITE_ID),
            author=user,
            title="Test Entry",
            raw_content="Test Content",
            content_format="html"
        )
        self.entry.publish()  # calls .save()

        self.unpublished_entry = Entry(
            site=Site.objects.get(pk=settings.SITE_ID),
            author=user,
            title="Test Entry",
            raw_content="Test Content",
            content_format="html",
            published=False
        )
        self.entry.save()

        self.feed = RSSEntriesFeed()

    def test_items(self):
        """Verify that ``.items()`` returns a queryset of published Entries"""
        published_entries = list(Entry.objects.filter(published=True))
        feed_entries = list(self.feed.items())
        self.assertEqual(feed_entries, published_entries)
        self.assertNotIn(self.unpublished_entry, feed_entries)

    def test_item_title(self):
        self.assertEqual(self.feed.item_title(self.entry), self.entry.title)

    def test_item_description(self):
        self.assertEqual(
            self.feed.item_description(self.entry),
            self.entry.content
        )


@override_settings(SITE_ID=1)
class TestAtomEntriesFeed(TestCase):
    """NOTE: This whole test is a duplicate of ``TestRSSEntriesFeed``."""
    def setUp(self):
        # Some setup (need a User and an Entry)
        User = get_user_model()
        username = 'blargg'
        user = User.objects.create(
            username=username,
            password='{0}@example.com'.format(username)
        )
        self.entry = Entry(
            site=Site.objects.get(pk=settings.SITE_ID),
            author=user,
            title="Test Entry",
            raw_content="Test Content",
            content_format="html"
        )
        self.entry.publish()  # calls .save()

        self.unpublished_entry = Entry(
            site=Site.objects.get(pk=settings.SITE_ID),
            author=user,
            title="Test Entry",
            raw_content="Test Content",
            content_format="html",
            published=False
        )
        self.entry.save()

        self.feed = AtomEntriesFeed()

    def test_items(self):
        """Verify that ``.items()`` returns a queryset of published Entries"""
        published_entries = list(Entry.objects.filter(published=True))
        feed_entries = list(self.feed.items())
        self.assertEqual(feed_entries, published_entries)
        self.assertNotIn(self.unpublished_entry, feed_entries)

    def test_item_title(self):
        self.assertEqual(self.feed.item_title(self.entry), self.entry.title)

    def test_item_description(self):
        self.assertEqual(
            self.feed.item_description(self.entry),
            self.entry.content
        )
